@@ -42,9 +42,6 @@ as.SpatialPointsDataFrame.ppp = function(from) {
 setAs("ppp", "SpatialPointsDataFrame", as.SpatialPointsDataFrame.ppp)
 
 as.SpatialGridDataFrame.ppp = function(from) {
-	# require(spatstat)
-    #if (!requireNamespace("spatstat", quietly = TRUE))
-	#	stop("package spatstat required for .SP2owin")
 	w = from$window
 	if (w$type != "mask")
 		stop("window is not of type mask")
@@ -60,7 +57,6 @@ as.SpatialGridDataFrame.ppp = function(from) {
 setAs("ppp", "SpatialGridDataFrame", as.SpatialGridDataFrame.ppp)
 
 as.SpatialGridDataFrame.im = function(from) {
-    # require(spatstat)
     offset = c(from$xrange[1] + 0.5 * from$xstep, from$yrange[1] + 
         0.5 * from$ystep)
     cellsize = c(diff(from$xrange)/from$dim[2], diff(from$yrange)/from$dim[1])
@@ -73,11 +69,9 @@ as.SpatialGridDataFrame.im = function(from) {
 setAs("im", "SpatialGridDataFrame", as.SpatialGridDataFrame.im)
 
 as.im.SpatialGridDataFrame = function(from) {
-    #require(spatstat)
-    if (!requireNamespace("spatstat", quietly = TRUE))
-		stop("package spatstat required for as.im.SpatialGridDataFrame")
+    check_spatstat("spatstat.geom")
     xi <- as.image.SpatialGridDataFrame(from)
-    spatstat::im(t(xi$z), xcol=xi$x, yrow=xi$y)
+    spatstat.geom::im(t(xi$z), xcol=xi$x, yrow=xi$y)
 }
 setAs("SpatialGridDataFrame", "im", as.im.SpatialGridDataFrame)
 
@@ -105,8 +99,7 @@ setAs("SpatialGridDataFrame", "im", as.im.SpatialGridDataFrame)
 
 as.im.RasterLayer <- function(from, factor.col.name = NULL) 
 {
-  if (!requireNamespace("spatstat", quietly = TRUE))
-    stop("package spatstat required for coercion")
+  check_spatstat("spatstat.geom")
   if (!requireNamespace("raster", quietly = TRUE))
     stop("package raster required for coercion")
   if (!raster::hasValues(from)) stop("values required in RasterLayer object")
@@ -136,13 +129,8 @@ as.im.RasterLayer <- function(from, factor.col.name = NULL)
   ## Assign dimensions to `val` as a matrix in raster layout:
   dim(val) <- dm
   ## Transform to spatstat format
-  val <- spatstat::transmat(val, from = list(x="-i", y="j"), to = "spatstat")
-  im <- spatstat::im(val, xcol=xx, yrow=yy)
+  val <- spatstat.geom::transmat(val, from = list(x="-i", y="j"), to = "spatstat")
+  im <- spatstat.geom::im(val, xcol=xx, yrow=yy)
   return(im)
 }
 
-
-#if (requireNamespace("spatstat", quietly = TRUE) && requireNamespace("raster", quietly = TRUE)) {
-#  setAs("RasterLayer", "im", as.im.RasterLayer)
-#}
-
